@@ -1,5 +1,5 @@
-/* -*- Mode: C; c-file-style: "bsd" -*- */
-/* main.c
+/* -*- Mode: C; c-file-style: "bsd" -*-
+ * main.c
  *       Copyright (C) 2001, 2002, 2003 Timo Schulz 
  *
  * This file is part of OpenCDK.
@@ -35,11 +35,13 @@
 #include "opencdk.h"
 #include "main.h"
 #include "packet.h"
+#include "cipher.h"
 
 
-#define DEFAULT_CIPHER_ALGO GCRY_CIPHER_CAST5
-#define DEFAULT_DIGEST_ALGO GCRY_MD_SHA1
+#define DEFAULT_CIPHER_ALGO CDK_CIPHER_CAST5
+#define DEFAULT_DIGEST_ALGO CDK_MD_SHA1
 
+#define SECMEM_SIZE 32768
 
 static void *(*alloc_func) (size_t n) = gcry_xmalloc;
 static void *(*alloc_secure_func) (size_t n) = gcry_malloc_secure;
@@ -49,7 +51,7 @@ static void (*free_func) (void *) = gcry_free;
 static int malloc_hooks = 0;
 static int secmem_init = 0;
 
-static CDK_LOG_FNC log_handler = NULL;
+static cdk_log_fnc_t log_handler = NULL;
 static void *log_handler_value = NULL;
 static int log_level = CDK_LOG_NONE;
 
@@ -74,7 +76,8 @@ cdk_strerror (int ec)
     case CDK_Inv_Packet:       return "Invalid packet";
     case CDK_Inv_Algo:         return "Invalid algorithm";
     case CDK_Not_Implemented:  return "This is not implemented yet";
-    case CDK_Gcry_Error:       return gcry_strerror (gcry_errno ());
+	/* FIXME: print the actual gcrypt error */
+    case CDK_Gcry_Error:       return "Gcrypt error";
     case CDK_Armor_Error:      return "ASCII armor error";
     case CDK_Armor_CRC_Error:  return "ASCII armored damaged (CRC error)";
     case CDK_MPI_Error:        return "Invalid or missformed MPI";
@@ -156,10 +159,10 @@ cdk_malloc (size_t size)
 void *
 cdk_calloc (size_t n, size_t m)
 {
-  void * p = calloc_func (n, m);
-  if (!p)
-    out_of_core (m);
-  return p;
+    void * p = calloc_func (n, m);
+    if (!p)
+        out_of_core (m);
+    return p;
 }
 
 
@@ -172,9 +175,10 @@ _secmem_init (size_t size)
     }
     if (secmem_init == 1)
         return;
-    if (size >= 65536)
-        size = 16384;
+    if (size >= SECMEM_SIZE)
+        size = SECMEM_SIZE;
     gcry_control (GCRYCTL_INIT_SECMEM, size, 0);
+    gcry_control (GCRYCTL_USE_SECURE_RNDPOOL);
     gcry_control (GCRYCTL_DISABLE_SECMEM_WARN);
     secmem_init = 1;
 }
@@ -183,8 +187,8 @@ _secmem_init (size_t size)
 static void
 _secmem_end (void)
 {
-  gcry_control (GCRYCTL_TERM_SECMEM);
-  secmem_init = 0;
+    gcry_control (GCRYCTL_TERM_SECMEM);
+    secmem_init = 0;
 }
 
 
@@ -195,15 +199,17 @@ cdk_salloc (size_t size, int clear)
     void * p;
   
     if (!secmem_init) {
-        _secmem_init (32768);
+        _secmem_init (SECMEM_SIZE);
         secmem_init = 1;
     }
     if (secmem_init == 1) {
         _secmem_init (0);
         secmem_init = 2;
     }
+    /*
     n += size;
-    /*_cdk_log_debug ("\ncdk_salloc (%d)\n", n);*/
+    _cdk_log_debug ("\ncdk_salloc (%d)\n", n);
+    */
     p = alloc_secure_func (size);
     if (!p)
         out_of_core (size);
@@ -279,7 +285,7 @@ _cdk_logv (int level, const char *fmt, va_list arg_ptr)
  * set a private handler for logging.
  */
 void
-cdk_set_log_handler (CDK_LOG_FNC logfnc, void * opaque)
+cdk_set_log_handler( cdk_log_fnc_t logfnc, void * opaque )
 {
     log_handler = logfnc;
     log_handler_value = opaque; 
@@ -342,7 +348,7 @@ getpass (const char * prompt)
 
 
 char *
-_cdk_passphrase_get( CDK_HD hd, const char * prompt )
+_cdk_passphrase_get( cdk_ctx_t hd, const char * prompt )
 {
     char * p, * pass = NULL;
     
@@ -374,38 +380,40 @@ _cdk_is_idea_available (void)
 
 
 static void
-handle_set_cipher (CDK_HD hd, int cipher)
+handle_set_cipher (cdk_ctx_t hd, int cipher)
 {
     if( !hd )
         return;
-    if (_cdk_cipher_test_algo (cipher))
+    if( cdk_cipher_test_algo( cipher ) )
         cipher = DEFAULT_CIPHER_ALGO;
     hd->cipher_algo = cipher;   
 }
 
 
 static void
-handle_set_digest (CDK_HD hd, int digest)
+handle_set_digest (cdk_ctx_t hd, int digest)
 {
     if( !hd )
         return;
-    if( _cdk_md_test_algo( digest ) )
+    if( cdk_md_test_algo( digest ) )
         digest = DEFAULT_DIGEST_ALGO;
     hd->digest_algo = digest;   
 }
 
 
 static void
-handle_set_s2k( CDK_HD hd, int mode, int digest, int cipher )
+handle_set_s2k( cdk_ctx_t hd, int mode, int digest, int cipher )
 {
     if( !hd )
         return;
-    if( _cdk_cipher_test_algo( cipher ) )
+    if( cdk_cipher_test_algo( cipher ) )
         cipher = DEFAULT_CIPHER_ALGO;
-    if( _cdk_md_test_algo( digest ) )
+    if( cdk_md_test_algo( digest ) )
         digest = DEFAULT_DIGEST_ALGO;
-    if( mode < 0 || mode > 3 )
-        mode = 3;
+    if( mode != CDK_S2K_SIMPLE
+        && mode != CDK_S2K_SALTED
+        && mode != CDK_S2K_ITERSALTED )
+        mode = CDK_S2K_ITERSALTED;
     hd->_s2k.mode = mode;
     hd->_s2k.digest_algo = digest;
     hd->_s2k.cipher_algo = cipher;
@@ -413,7 +421,7 @@ handle_set_s2k( CDK_HD hd, int mode, int digest, int cipher )
 
 
 static void
-handle_set_compat( CDK_HD hd, int val )
+handle_set_compat( cdk_ctx_t hd, int val )
 {
     if( !hd )
         return;
@@ -424,15 +432,15 @@ handle_set_compat( CDK_HD hd, int val )
     hd->opt.rfc1991 = val == -1? 1: 0;
     hd->compress.algo = CDK_COMPRESS_ZIP;
     hd->compress.level = -1;
-    hd->cipher_algo = val == -1? GCRY_CIPHER_IDEA : DEFAULT_CIPHER_ALGO;
-    hd->digest_algo = val == -1? GCRY_MD_MD5: DEFAULT_DIGEST_ALGO;
+    hd->cipher_algo = val == -1? CDK_CIPHER_IDEA : DEFAULT_CIPHER_ALGO;
+    hd->digest_algo = val == -1? CDK_MD_MD5: DEFAULT_DIGEST_ALGO;
     if( val == -1 )
         handle_set_s2k( hd, 0, hd->digest_algo, hd->cipher_algo );   
 }
 
 
 static void
-handle_set_compress( CDK_HD hd, int algo, int level )
+handle_set_compress( cdk_ctx_t hd, int algo, int level )
 {
     if( !hd )
         return;
@@ -459,7 +467,7 @@ handle_set_compress( CDK_HD hd, int algo, int level )
  * Perform various control operations for the current session.
  **/
 int
-cdk_handle_control( CDK_HD hd, int action, int cmd, ... )
+cdk_handle_control( cdk_ctx_t hd, int action, int cmd, ... )
 {
     va_list arg_ptr;
     int set = action == CDK_CTLF_SET, val = 0;
@@ -569,9 +577,9 @@ cdk_handle_control( CDK_HD hd, int action, int cmd, ... )
  * create a new session handle.
  **/
 int
-cdk_handle_new( CDK_HD * r_ctx )
+cdk_handle_new( cdk_ctx_t * r_ctx )
 {
-    CDK_HD c;
+    cdk_ctx_t c;
   
     if( !r_ctx )
         return CDK_Inv_Value;
@@ -609,7 +617,7 @@ cdk_handle_new( CDK_HD * r_ctx )
  * secret keyring and the right handle is set.
  **/
 void
-cdk_handle_set_keydb( CDK_HD hd, CDK_KEYDB_HD db )
+cdk_handle_set_keydb( cdk_ctx_t hd, cdk_keydb_hd_t db )
 {
     if( !hd )
         return;
@@ -627,8 +635,8 @@ cdk_handle_set_keydb( CDK_HD hd, CDK_KEYDB_HD db )
  *
  * Return the keydb handle from the session handle.
  **/
-CDK_KEYDB_HD
-cdk_handle_get_keydb( CDK_HD hd, int type )
+cdk_keydb_hd_t
+cdk_handle_get_keydb( cdk_ctx_t hd, int type )
 {
     if( !hd )
         return NULL;
@@ -649,7 +657,7 @@ cdk_handle_get_keydb( CDK_HD hd, int type )
  * set the callback for filter operations.
  **/
 void
-cdk_handle_set_callback (CDK_HD hd,
+cdk_handle_set_callback (cdk_ctx_t hd,
                          void (*cb) (void * opaque, int type, const char * s),
                          void * cb_value)
 {
@@ -668,7 +676,7 @@ cdk_handle_set_callback (CDK_HD hd,
  *
  * set the passphrase callback.
  **/
-void cdk_handle_set_passphrase_cb( CDK_HD hd,
+void cdk_handle_set_passphrase_cb( cdk_ctx_t hd,
                                    char *(*cb) (void *opa, const char *prompt),
                                    void * cb_value )
 {
@@ -686,7 +694,7 @@ void cdk_handle_set_passphrase_cb( CDK_HD hd,
  * free the main handle.
  **/
 void
-cdk_handle_free( CDK_HD hd )
+cdk_handle_free( cdk_ctx_t hd )
 {
     if( !hd )
         return;
