@@ -22,6 +22,7 @@
 #ifndef CDK_MAIN_H
 #define CDK_MAIN_H
 
+#include <config.h>
 #include <gcrypt.h>
 #include "md.h"
 #include "types.h"
@@ -74,7 +75,13 @@ u32 _cdk_buftou32 (const byte * buf);
 void _cdk_u32tobuf (u32 u, byte * buf);
 const char * _cdk_memistr (const char * buf, size_t buflen, const char * sub);
 void _cdk_vasprintf_free (void * p);
-    
+
+#ifndef HAVE_VASPRINTF
+int _cdk_vasprintf ( char **result, const char *format, va_list args);
+#else
+# define _cdk_vasprintf vasprintf
+#endif
+
 /*-- pubkey.c --*/
 u32 _cdk_pkt_get_keyid( cdk_packet_t pkt, u32 * keyid );
 int _cdk_pkt_get_fingerprint( cdk_packet_t pkt, byte * fpr );
